import string
import base64
import numpy


class QueryData(object):
    def __init__(self, data: list):
        self.data = data

    def fetch_one(self) -> string:
        return self.data[0]

    def fetch_many(self, size: int) -> list:
        return self.data[0:size]

    def fetch_all(self):
        return self.data


class Field(object):
    def __init__(self):
        self.name = None
        self.field_type = None
        self.precision = None
        self.scale = None
        self.length = None
        self.nullable = None

    def set_name(self, name):
        self.name = name

    def set_type(self, type):
        self.field_type = type

    def set_precision(self, precision):
        self.precision = precision

    def set_scale(self, scale):
        self.scale = scale

    def set_length(self, length):
        self.length = length

    def set_nullable(self, nullable):
        self.nullable = nullable


class QueryResult(object):
    def __init__(self, total_msg: string):
        self.data = None
        self.state = None
        self.total_row_count = 0
        self.total_msg = total_msg
        self.schema = []
        self._parse_result_data()

    def _parse_field(self, field: str, schema_field: Field):
        schema_field.set_name(field['name'])
        if field['type'].__contains__('charTypeInfo'):
            schema_field.set_type(field['type']['category'])
            schema_field.set_nullable(str(field['type']['nullable']) != 'False')
            schema_field.set_length(field['type']['charTypeInfo']['length'])
        elif field['type'].__contains__('decimalTypeInfo'):
            schema_field.set_type(field['type']['category'])
            schema_field.set_nullable(str(field['type']['nullable']) == 'true')
            schema_field.set_precision(field['type']['decimalTypeInfo']['precision'])
            schema_field.set_scale(field['type']['decimalTypeInfo']['scale'])
        else:
            schema_field.set_type(field['type']['category'])
            schema_field.set_nullable(str(field['type']['nullable']) == 'true')

    def get_result_state(self) -> string:
        return self.total_msg['status']['state']

    def _parse_result_data(self):
        self.state = self.total_msg['status']['state']
        result_data = self.total_msg['resultSet']['data']['data'][0]
        decode_data = base64.b64decode(result_data).strip().decode('utf-8').replace('\\', '').replace('\\n', '')
        data_list = decode_data.split('\n')
        self.total_row_count = len(data_list)
        fields = self.total_msg['resultSet']['metadata']['fields']
        for field in fields:
            schema_field = Field()
            self._parse_field(field, schema_field)
            self.schema.append(schema_field)
        result_data = []
        for row in data_list:
            data_list = []
            for value in row.split(','):
                data_list.append(value)
            result_data.append(data_list)
        self.data = QueryData(result_data)
